package com.jpsoft.picc.modules.admin.controller;

import com.jpsoft.picc.modules.common.dto.MessageResult;
import com.jpsoft.picc.modules.common.utils.DES3;
import com.jpsoft.picc.modules.sys.entity.User;
import com.jpsoft.picc.modules.sys.service.UserService;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.security.Keys;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestAttribute;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpSession;
import java.security.Key;
import java.util.Base64;
import java.util.Date;

@RestController
@Api(description = "用户登录、登出")
public class JwtsUserController {
    @Value("${jwt.secret}")
    private String jwtSecret;

    @Autowired
    private UserService userService;

    @PostMapping("/login")
    @ApiOperation(value="登录获取token,在swagger ui中获取token时将写入session，调用其它接口时不用再设置header")
    @ApiImplicitParams({
            @ApiImplicitParam(name="userName", paramType="query", required=true, value="用户名"),
            @ApiImplicitParam(name="password", paramType="query", required=true, value="密码")
    })
    public MessageResult<String> login(String userName, String password, @ApiIgnore HttpSession session){
        MessageResult<String> messageResult = new MessageResult<>();

        try {
            User user = userService.findByUserName(userName);

            DES3 des3 = new DES3();

            String passwordEnc = des3.encrypt(jwtSecret,password);

            if(user!=null && passwordEnc.equals(user.getPassword())){
                //生成token
                Date now = new Date();
                long expiration = now.getTime() + 3600 * 6000; //6个小时后，该客户端的token过期

                byte[] privateKey = Base64.getDecoder().decode(jwtSecret);

                Key key = Keys.hmacShaKeyFor(privateKey);

                String token = Jwts.builder()
                        .setSubject(user.getId())
                        //设置自定义claims后，setSubject值将失效
//               .setClaims(extraInfo)
                        .signWith(key)
                        .setExpiration(new Date(expiration))
                        .compact();

                session.setAttribute("token","Bearer " + token);

                messageResult.setResult(true);
                messageResult.setData("Bearer " + token);
            }
            else{
                messageResult.setResult(false);
                messageResult.setMessage("用户不存在或密码错误！");
            }

        }
        catch(Exception ex){
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @GetMapping("/userInfo")
    public MessageResult<User> userInfo(@RequestAttribute String subject){
        MessageResult<User> messageResult = new MessageResult<>();

        try {
            User user = userService.get(subject);

            messageResult.setResult(true);
            messageResult.setData(user);
        }
        catch(Exception ex){
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }
}
