package com.jpsoft.picc.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.picc.modules.base.entity.Company;
import com.jpsoft.picc.modules.common.utils.PojoUtils;
import com.jpsoft.picc.modules.common.dto.Sort;
import com.jpsoft.picc.modules.common.dto.MessageResult;
import com.jpsoft.picc.modules.base.service.CompanyService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;

@RestController
@RequestMapping("/company")
@Api(description = "企业信息")
public class CompanyController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private CompanyService companyService;

    @ApiOperation(value="获取信息")
    @GetMapping("detail")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "companyId",value = "企业编号", required = true, paramType = "query")
    })
    public MessageResult<Company> detail(String companyId){
        MessageResult<Company> msgResult = new MessageResult<>();

        try {
            Company company = companyService.get(companyId);

            if (company != null) {
                msgResult.setResult(true);
                msgResult.setData(company);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

//    @ApiOperation(value="更新企业信息")
//    @PostMapping("update")
    @Deprecated
    public MessageResult<Company> update(@RequestBody Company company,@RequestAttribute String subject){
        MessageResult<Company> msgResult = new MessageResult<>();

        try {
            company.setUpdateBy(subject);
            company.setUpdateTime(new Date());
            
            int affectCount = companyService.update(company);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(company);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

//	@ApiOperation(value="删除")
//  @PostMapping("delete/{id}")
    @Deprecated
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            Company company = companyService.get(id);
            company.setDelFlag(true);
            company.setUpdateBy(subject);
            company.setUpdateTime(new Date());

            int affectCount = companyService.update(company);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


//    @ApiOperation(value="批量删除")
//    @PostMapping("batchDelete")
    @Deprecated
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                Company company = companyService.get(id);
                company.setDelFlag(true);
                company.setUpdateBy(subject);
                company.setUpdateTime(new Date());

                affectCount += companyService.update(company);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "name",value = "企业名称", paramType = "form"),
            @ApiImplicitParam(name = "legalName",value = "法人代表", paramType = "form"),
            @ApiImplicitParam(name = "usccCode",value = "证件编号", paramType = "form"),
            @ApiImplicitParam(name = "subject",value = "当前用户编号（系统自动获取，不用填写）")
    })
    public MessageResult<Map> pageList(
            String name,String legalName,String usccCode,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("create_time","asc"));

        if (StringUtils.isNotEmpty(name)) {
            searchParams.put("name","%" + name + "%");
        }

        if (StringUtils.isNotEmpty(legalName)) {
            searchParams.put("legalName","%" + legalName + "%");
        }

        if (StringUtils.isNotEmpty(usccCode)) {
            searchParams.put("usccCode","%" + usccCode + "%");
        }

        Page<Company> page = companyService.pageSearch(searchParams,pageIndex,pageSize,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }
}
