package com.jpsoft.picc.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.picc.modules.base.entity.InsuranceAgent;
import com.jpsoft.picc.modules.common.utils.PojoUtils;
import com.jpsoft.picc.modules.common.dto.Sort;
import com.jpsoft.picc.modules.common.dto.MessageResult;
import com.jpsoft.picc.modules.base.service.InsuranceAgentService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;

@RestController
@RequestMapping("/insuranceAgent")
@Api(description = "保险代理人")
public class InsuranceAgentController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private InsuranceAgentService insuranceAgentService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<InsuranceAgent> create(){
        MessageResult<InsuranceAgent> msgResult = new MessageResult<>();

        InsuranceAgent insuranceAgent = new InsuranceAgent();

        msgResult.setData(insuranceAgent);
        msgResult.setResult(true);

        return msgResult;
    }
    
    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<InsuranceAgent> add(@RequestBody InsuranceAgent insuranceAgent,@RequestAttribute String subject){
        MessageResult<InsuranceAgent> msgResult = new MessageResult<>();

        try {
            insuranceAgent.setId(UUID.randomUUID().toString());
            insuranceAgent.setDelFlag(false);
            insuranceAgent.setCreateBy(subject);
            insuranceAgent.setCreateTime(new Date());
            
            int affectCount = insuranceAgentService.insert(insuranceAgent);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(insuranceAgent);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<InsuranceAgent> edit(@PathVariable("id") String id){
        MessageResult<InsuranceAgent> msgResult = new MessageResult<>();

        try {
            InsuranceAgent insuranceAgent = insuranceAgentService.get(id);

            if (insuranceAgent != null) {
                msgResult.setResult(true);
                msgResult.setData(insuranceAgent);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<InsuranceAgent> update(@RequestBody InsuranceAgent insuranceAgent,@RequestAttribute String subject){
        MessageResult<InsuranceAgent> msgResult = new MessageResult<>();

        try {
            insuranceAgent.setUpdateBy(subject);
            insuranceAgent.setUpdateTime(new Date());
            
            int affectCount = insuranceAgentService.update(insuranceAgent);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(insuranceAgent);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            InsuranceAgent insuranceAgent = insuranceAgentService.get(id);
            insuranceAgent.setDelFlag(true);
            insuranceAgent.setUpdateBy(subject);
            insuranceAgent.setUpdateTime(new Date());

            int affectCount = insuranceAgentService.update(insuranceAgent);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                InsuranceAgent insuranceAgent = insuranceAgentService.get(id);
                insuranceAgent.setDelFlag(true);
                insuranceAgent.setUpdateBy(subject);
                insuranceAgent.setUpdateTime(new Date());

                affectCount += insuranceAgentService.update(insuranceAgent);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String id,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("create_time","desc"));

        if (StringUtils.isNotEmpty(id)) {
            searchParams.put("id","%" + id + "%");
        }

        Page<InsuranceAgent> page = insuranceAgentService.pageSearch(searchParams,pageIndex,pageSize,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }


    @ApiOperation(value="所有代理人列表")
    @RequestMapping(value = "list",method = RequestMethod.POST)
    public MessageResult<List<InsuranceAgent>> list(){

        MessageResult<List<InsuranceAgent>> msgResult = new MessageResult<>();

        List<InsuranceAgent> list = insuranceAgentService.list();

        msgResult.setResult(true);
        msgResult.setData(list);

        return msgResult;
    }
}
