package com.jpsoft.picc.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.picc.modules.base.entity.InsuranceJobs;
import com.jpsoft.picc.modules.common.utils.PojoUtils;
import com.jpsoft.picc.modules.common.dto.Sort;
import com.jpsoft.picc.modules.common.dto.MessageResult;
import com.jpsoft.picc.modules.base.service.InsuranceJobsService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;

@RestController
@RequestMapping("/insuranceJobs")
@Api(description = "险种关联行业收费标准")
public class InsuranceJobsController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private InsuranceJobsService insuranceJobsService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<InsuranceJobs> create(){
        MessageResult<InsuranceJobs> msgResult = new MessageResult<>();

        InsuranceJobs insuranceJobs = new InsuranceJobs();

        msgResult.setData(insuranceJobs);
        msgResult.setResult(true);

        return msgResult;
    }
    
    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<InsuranceJobs> add(@RequestBody InsuranceJobs insuranceJobs,@RequestAttribute String subject){
        MessageResult<InsuranceJobs> msgResult = new MessageResult<>();

        try {
            insuranceJobs.setId(UUID.randomUUID().toString());
            insuranceJobs.setDelFlag(false);
            insuranceJobs.setCreateBy(subject);
            insuranceJobs.setCreateTime(new Date());
            
            int affectCount = insuranceJobsService.insert(insuranceJobs);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(insuranceJobs);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<InsuranceJobs> edit(@PathVariable("id") String id){
        MessageResult<InsuranceJobs> msgResult = new MessageResult<>();

        try {
            InsuranceJobs insuranceJobs = insuranceJobsService.get(id);

            if (insuranceJobs != null) {
                msgResult.setResult(true);
                msgResult.setData(insuranceJobs);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<InsuranceJobs> update(@RequestBody InsuranceJobs insuranceJobs,@RequestAttribute String subject){
        MessageResult<InsuranceJobs> msgResult = new MessageResult<>();

        try {
            insuranceJobs.setUpdateBy(subject);
            insuranceJobs.setUpdateTime(new Date());
            
            int affectCount = insuranceJobsService.update(insuranceJobs);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(insuranceJobs);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            InsuranceJobs insuranceJobs = insuranceJobsService.get(id);
            insuranceJobs.setDelFlag(true);
            insuranceJobs.setUpdateBy(subject);
            insuranceJobs.setUpdateTime(new Date());

            int affectCount = insuranceJobsService.update(insuranceJobs);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                InsuranceJobs insuranceJobs = insuranceJobsService.get(id);
                insuranceJobs.setDelFlag(true);
                insuranceJobs.setUpdateBy(subject);
                insuranceJobs.setUpdateTime(new Date());

                affectCount += insuranceJobsService.update(insuranceJobs);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String id,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("id_","asc"));

        if (StringUtils.isNotEmpty(id)) {
            searchParams.put("id","%" + id + "%");
        }

        Page<InsuranceJobs> page = insuranceJobsService.pageSearch(searchParams,pageIndex,pageSize,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }



    @ApiOperation(value="险种关联行业类型列表")
    @RequestMapping(value = "list",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name="definitionId",value = "险种信息ID",required = true,paramType = "query")
    })
    public MessageResult<List<InsuranceJobs>> list(String definitionId, @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<List<InsuranceJobs>> msgResult = new MessageResult<>();

        List<InsuranceJobs> list = insuranceJobsService.listByDefinitionId(definitionId);

        msgResult.setResult(true);
        msgResult.setData(list);

        return msgResult;
    }
}
