package com.jpsoft.picc.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.picc.modules.base.entity.Jobs;
import com.jpsoft.picc.modules.common.utils.PojoUtils;
import com.jpsoft.picc.modules.common.dto.Sort;
import com.jpsoft.picc.modules.common.dto.MessageResult;
import com.jpsoft.picc.modules.base.service.JobsService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;

@RestController
@RequestMapping("/jobs")
@Api(description = "行业/职业类型信息")
public class JobsController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private JobsService jobsService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<Jobs> create(){
        MessageResult<Jobs> msgResult = new MessageResult<>();

        Jobs jobs = new Jobs();

        msgResult.setData(jobs);
        msgResult.setResult(true);

        return msgResult;
    }
    
    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<Jobs> add(@RequestBody Jobs jobs,@RequestAttribute String subject){
        MessageResult<Jobs> msgResult = new MessageResult<>();

        try {
            jobs.setId(UUID.randomUUID().toString());
            jobs.setDelFlag(false);
            jobs.setCreateBy(subject);
            jobs.setCreateTime(new Date());
            
            int affectCount = jobsService.insert(jobs);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(jobs);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<Jobs> edit(@PathVariable("id") String id){
        MessageResult<Jobs> msgResult = new MessageResult<>();

        try {
            Jobs jobs = jobsService.get(id);

            if (jobs != null) {
                msgResult.setResult(true);
                msgResult.setData(jobs);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<Jobs> update(@RequestBody Jobs jobs,@RequestAttribute String subject){
        MessageResult<Jobs> msgResult = new MessageResult<>();

        try {
            jobs.setUpdateBy(subject);
            jobs.setUpdateTime(new Date());
            
            int affectCount = jobsService.update(jobs);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(jobs);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            Jobs jobs = jobsService.get(id);
            jobs.setDelFlag(true);
            jobs.setUpdateBy(subject);
            jobs.setUpdateTime(new Date());

            int affectCount = jobsService.update(jobs);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                Jobs jobs = jobsService.get(id);
                jobs.setDelFlag(true);
                jobs.setUpdateBy(subject);
                jobs.setUpdateTime(new Date());

                affectCount += jobsService.update(jobs);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String id,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("create_time","desc"));

        if (StringUtils.isNotEmpty(id)) {
            searchParams.put("id","%" + id + "%");
        }

        Page<Jobs> page = jobsService.pageSearch(searchParams,pageIndex,pageSize,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }


    @ApiOperation(value="所有行业类型列表")
    @RequestMapping(value = "list",method = RequestMethod.POST)
    public MessageResult<List<Jobs>> list(){

        MessageResult<List<Jobs>> msgResult = new MessageResult<>();

        List<Jobs> list = jobsService.list();

        msgResult.setResult(true);
        msgResult.setData(list);

        return msgResult;
    }
}
