package com.jpsoft.picc.modules.business.controller;

import com.github.pagehelper.Page;
import com.jpsoft.picc.modules.common.utils.PojoUtils;
import com.jpsoft.picc.modules.common.dto.Sort;
import com.jpsoft.picc.modules.common.dto.MessageResult;
import com.jpsoft.picc.modules.business.entity.InsurancePolicyRecord;
import com.jpsoft.picc.modules.business.service.InsurancePolicyRecordService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/insurancePolicyRecord")
@Api(description = "每月投保单审核流水")
public class InsurancePolicyRecordController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private InsurancePolicyRecordService insurancePolicyRecordService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<InsurancePolicyRecord> create(){
        MessageResult<InsurancePolicyRecord> msgResult = new MessageResult<>();

        InsurancePolicyRecord insurancePolicyRecord = new InsurancePolicyRecord();

        msgResult.setData(insurancePolicyRecord);
        msgResult.setResult(true);

        return msgResult;
    }
    
    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<InsurancePolicyRecord> add(@RequestBody InsurancePolicyRecord insurancePolicyRecord,@RequestAttribute String subject){
        MessageResult<InsurancePolicyRecord> msgResult = new MessageResult<>();

        try {
            insurancePolicyRecord.setId(UUID.randomUUID().toString());
            insurancePolicyRecord.setDelFlag(false);
            insurancePolicyRecord.setCreateBy(subject);
            insurancePolicyRecord.setCreateTime(new Date());
            
            int affectCount = insurancePolicyRecordService.insert(insurancePolicyRecord);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(insurancePolicyRecord);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<InsurancePolicyRecord> edit(@PathVariable("id") String id){
        MessageResult<InsurancePolicyRecord> msgResult = new MessageResult<>();

        try {
            InsurancePolicyRecord insurancePolicyRecord = insurancePolicyRecordService.get(id);

            if (insurancePolicyRecord != null) {
                msgResult.setResult(true);
                msgResult.setData(insurancePolicyRecord);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<InsurancePolicyRecord> update(@RequestBody InsurancePolicyRecord insurancePolicyRecord,@RequestAttribute String subject){
        MessageResult<InsurancePolicyRecord> msgResult = new MessageResult<>();

        try {
            insurancePolicyRecord.setUpdateBy(subject);
            insurancePolicyRecord.setUpdateTime(new Date());
            
            int affectCount = insurancePolicyRecordService.update(insurancePolicyRecord);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(insurancePolicyRecord);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            InsurancePolicyRecord insurancePolicyRecord = insurancePolicyRecordService.get(id);
            insurancePolicyRecord.setDelFlag(true);
            insurancePolicyRecord.setUpdateBy(subject);
            insurancePolicyRecord.setUpdateTime(new Date());

            int affectCount = insurancePolicyRecordService.update(insurancePolicyRecord);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                InsurancePolicyRecord insurancePolicyRecord = insurancePolicyRecordService.get(id);
                insurancePolicyRecord.setDelFlag(true);
                insurancePolicyRecord.setUpdateBy(subject);
                insurancePolicyRecord.setUpdateTime(new Date());

                affectCount += insurancePolicyRecordService.update(insurancePolicyRecord);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String id,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("id_","asc"));

        if (StringUtils.isNotEmpty(id)) {
            searchParams.put("id","%" + id + "%");
        }

        Page<InsurancePolicyRecord> page = insurancePolicyRecordService.pageSearch(searchParams,pageIndex,pageSize,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }



    @ApiOperation(value="企业投保审批历史记录")
    @RequestMapping(value = "historyList",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name="policy_id",value = "每月投保单申请ID",required = true,paramType = "query")
    })
    public MessageResult<List<InsurancePolicyRecord>> historyList(String policy_id,@RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<List<InsurancePolicyRecord>> msgResult = new MessageResult<>();

        List<InsurancePolicyRecord> list = insurancePolicyRecordService.historyList(policy_id);

        msgResult.setResult(true);
        msgResult.setData(list);

        return msgResult;
    }
}
